
library(lme4)
library(ggplot2)
library(car)
library(lmerTest)

setwd()
load("MetaRegression.Rda")


###############################################################################
###############################################################################
### On your own: 
### Do some data exploration (e.g pairs plots, correlations, simple regression models)






###############################################################################

##### Let's look at how the study effects these relationships

ggplot(dat, aes(y=y, x=Dose, colour=study)) + geom_point() + geom_line()

## Does the data have homegenous variance?
ggplot(dat, aes(y = y, x = study)) + geom_boxplot(aes(group = study))

## Now let's explore a mixed model without accounting for heterogeneity

### Full model with random intercept

FullMod = lmer(y ~ Dose +  DoseConc + BW + (1|study), data = dat)
summary(FullMod)


#### Select fixed effects fit with ML
FullModML = lmer(y ~ Dose + DoseConc + BW + (1|study), data = dat, REML = FALSE)
RedModML1 = lmer(y ~ Dose +  BW + (1|study), data = dat, REML = FALSE)
RedModML2 = lmer(y ~ Dose + (1|study), data = dat, REML = FALSE)
RedModML3 = lmer(y ~ 1 +(1|study), data = dat, REML = FALSE)

anova(FullModML, RedModML1)
anova(RedModML1, RedModML2)
anova(RedModML2, RedModML3)

### Let's look at some model evaluation of the selected model

Mod1 = lmer(y ~ Dose + (1|study), data = dat, REML = TRUE)

#get residuals
res = residuals(Mod1)

#look at residuals all together
boxplot(res)
dat$Res = res

## plot residuals separately
ggplot(dat, aes(y = Res, x = study)) + geom_boxplot(aes(group = study))


### plot residuals against observed and independent variabls
plot(res ~ dat$BW)
plot(res ~ dat$Dose)
plot(res ~ predict(Mod1))



##############################################################################
##############################################################################

## Let's account for the heterogenity

HetMod1 = lmer(y ~ Dose + (1|study), data = dat, weights = 1/SEI^2 )
summary(HetMod1)



### Now let's look at the residuals again

dat$Res = residuals(HetMod1)
plot(Res ~ y, data = dat)

#######################################################################
#######################################################################
### On your own:
### Interpretation - how do the variance components and SE of the fixed effects
### differ between the model with and without heterogenity?

Mod1
HetMod1


######################################################################



#### Now let's investigate if we need a random effect on the slope

#### let's fit simple regression models indepdently for each of the studies to see
#### if the slopes are different across studies

mod.list = lmList(y ~ Dose |study, data = dat)
summary(mod.list)
CI<- confint(mod.list)
plot(CI)

### Should there be a random effect of slope?



### Let's fit a random slope effect

HetMod2 = lmer(y ~ Dose + (Dose|study), data = dat, weights = 1/SEI^2 )
summary(HetMod2)


### How can we test the random effects?



RandomTestMod = as_lmerModLmerTest(HetMod2)

ranova(RandomTestMod, reduce = TRUE) # removes just the slope effect
ranova(RandomTestMod, reduce = FALSE) # removes all the effects


### let's add a random effect that shouldn't be there just to see

## create an unrelated grouping factor and add it to the dataset
set.seed(52883)
rand.fact = as.factor(sample(1:7, 150, replace = TRUE))
dat$G2 = rand.fact

falseMod = as_lmerModLmerTest(lmer(y ~ Dose +  (1|G2) + (1|study), 
                                   data = dat, weights = 1/SEI^2))

ranova(falseMod, reduce = TRUE)




###################################################################
###################################################################
### model Evaluation
###################################################################



#What about the heterogeneity? How do we assess it's degree and influence on the model

HetMod0 <- lmer(y ~ 1 + (1|study) , weights = 1/SEI^2, data = dat )
summary(HetMod0)

summary(HetMod2)



## Subtract the study variance of the full model from the model w/o
## the fixed effects to find the percent of heterogeneity that was reduced by including 
## the appropriate fixed effects - analagous to I2 in metafor




### Funnel plot
### create funnel plot manually


### Because the observed values are so large in comparison to the variances, used the 
### standardized predicted values

y.hat = predict(HetMod2)
y.mean = mean(y.hat) 
sd.y = sd(y.hat)
standard.y.hat = (y.hat - y.mean)/sd.y

#Set the limits of the y range to the reverse order of the sei
ylims = rev(range(dat$SEI))
ylims[2] = 0

plot(dat$SEI ~ standard.y.hat, ylim = ylims, xlim = c( -2.5,  2.5), 
     xlab = "Response", 
     ylab = "SEi")
abline(v = 0)

#Store a vector of values that spans the range from 0
#to the max value of standard error in your dataset.
se.seq=seq(0, max(dat$SEI), 0.001)

#Compute vectors of the lower-limit and upper limit values for
#the 95% CI region
ll95 = 0-(se.seq)
ul95 = 0+(se.seq)

## add lines
lines(se.seq~ll95)
lines(se.seq~ul95)
abline(v = y.mean)
